#Import der benötigten Module
#zum Zugriff auf den Sensor DS18B20
#und der Pins des Pi Pico
import machine, onewire, ds18x20, time


#Import der benötigten Module
#für den Aufbau der WiFi-Verbindung
import network
import socket

#Modul zum verarbeiten des JSON
import json

ssid = 'FRITZBox7590GI24'
password = '22894580214767401850'

#der Sensor ist am GPIO16 angeschlossen
sensorPin = machine.Pin(16)
#initialisieren eines Objektes vom Typ DS18X20
ds18b20Sensor = ds18x20.DS18X20(onewire.OneWire(sensorPin))

#anlegen eines Feldes für die IDs der Sensoren
roms = None

#anlegen eines Feldes für die WiFi-Verbindung
wlan = None

s = None
data = []

#Feld für die gespeicherte Daten vom Pi Pico
storedData = {}

#aktueller Zeitstempel
actualTime = 0
#Zeitstempel der letzten Ausführung
lastReadSensordata = 0
#Intervall in Sekunden in welchem
#die Sensordaten gelesen werden sollen
readSensordataIntervall = 30

#Feld zum speichern der CssStyles
#die Daten ändern sich nicht und
#daher werden diese einmalig eingelesen
cssStyles = None

def doConnect():
    global wlan, s
    
    wlan=network.WLAN(network.STA_IF)
    wlan.active(True)
    
    #Wenn die Verbindung nicht erstellt wurde, dann...
    if not wlan.isconnected():
        #Aufbau der WiFi-Verbindung mit den Daten
        wlan.connect(ssid, password)
        #solange die Verbindung noch nicht hergestellt wurde
        #dann soll diese Schleife laufen und einen Punkt ausgeben
        while not wlan.isconnected():
            #einen Punkt ausgeben ohne Zeilenumbruch!
            print(".", end="")
            #kleine Pause von 250 Millisekunden
            time.sleep(0.250)
        print("")
        
    #Wenn die WiFi-Verbindung erfolgreich aufgebaut wurde,
    #dann soll auf der seriellen Schnittstelle der Text ausgegeben werden
    print('Verbindung erfolgreich aufgebaut!')
    
    status = wlan.ifconfig()
    print('IP-Adresse: ' + status[0])
    addr = socket.getaddrinfo('0.0.0.0', 80)[0][-1]
    s = socket.socket()
    s.bind(addr)
    s.listen(1)
    
def readSensordata():
    result = []
    #auslesen der Sensorwerte
    ds18b20Sensor.convert_temp()
    #nach dem aufruf der Funktion "convert_temp()"
    #soll man gem. Dokumentation 750ms warten
    time.sleep_ms(750)
    #erzeugen eines Indexes
    #Schleife über die gefundenen IDs / Sensoren
    idx = 0
    for rom in roms:
        tempValue = round(ds18b20Sensor.read_temp(rom),2)
        appendDataToJson(tempValue, idx)
        result.append(tempValue)
        idx = idx + 1        
    
    storeDataToJson()
    
    return result

def appendDataToJson(tempValue, idx):
    tempList = storedData["sensors"][idx]["values"]
    if len(tempList)>=10:
        tempList.insert(0, tempValue)
        tempList.pop()
    else:
        tempList.append(tempValue)
    
    maxTemp = storedData["sensors"][idx]["max"]
    maxTemp = max(maxTemp, tempValue)
    storedData["sensors"][idx]["max"] = maxTemp
    
    minTemp = storedData["sensors"][idx]["min"]
    minTemp = min(minTemp, tempValue)
    storedData["sensors"][idx]["min"] = minTemp
    
    averageTemp = storedData["sensors"][idx]["average"]
    averageTemp = round(sum(tempList) / len(tempList),2)
    storedData["sensors"][idx]["average"] = averageTemp 

def readJsonToData():
    global storedData
    file = open('data.json')
    storedData = json.load(file)
    print(storedData)
    file.close()

def storeDataToJson():
    global storedData
    with open("data.json", "w") as outfile:
        json.dump(storedData, outfile)

def createHtmlTable(data):
    htmlTable = """
        <table border='1' style='border-collapse: collapse;'>
            <thead>
                <tr>
                      <th>Sensor</th>
                      <th>Wert</th>
                      <th>min</th>
                      <th>max</th>
                      <th>durchschnitt</th>
                </tr>
            </thead>
            <tbody>
                {bodyRows}
            </tbody>
        </table>
    """
    
    idx = 0
    rows = ""
    for value in data:
        idx = idx + 1
        minValue = storedData["sensors"][idx-1]["min"]
        maxValue = storedData["sensors"][idx-1]["max"]
        average = storedData["sensors"][idx-1]["average"]
        
        rows += "<tr><td>{sensor}</td><td>{wert}</td><td>{minValue}</td><td>{maxValue}</td><td>{average}</td></tr>".format(
            sensor=str(idx),
            wert=str(value)+"&nbsp;&deg;C",
            minValue=str(minValue)+"&nbsp;&deg;C",
            maxValue=str(maxValue)+"&nbsp;&deg;C",
            average=str(average)+"&nbsp;&deg;C")
        
    return htmlTable.format(bodyRows=rows)

def getCssStyles():
    global cssStyles
    if cssStyles == None:
        with open('styles.css', "r") as file:
            cssStyles = file.read()
            file.close()
    
    return cssStyles

def getWebsite(dataTbl):
    with open('website.html', "r") as file:
        website = file.read()
        file.close()
    
    return website.format(cssStyle= getCssStyles(),dataTable=dataTbl)

def deliverWebsite(website):
    global s
    cl, addr = s.accept()
    cl.send('HTTP/1.0 200 OK\r\nContent-type: text/html\r\n\r\n')
    cl.send(website)
    cl.close()

def setup():
    #Zugriff auf die globalen Variablen roms, wifi
    global roms
    #auslesen der IDs der Sensoren
    roms = ds18b20Sensor.scan()
    #ausgeben der Anzahl der Sensoren auf
    #der seriellen Schnittstelle
    print('Anzahl gefundener Sensoren: ', len(roms))
    
    readJsonToData()
    doConnect()

def main():
    #Zugriff auf die Globale Variable roms
    global roms, data, lastReadSensordata
    #starten einer Endlosschleife, ...
    while True:
      #nur alle 30 Sekunden neue Sensordaten lesen
      actualTime = time.time()
      if (lastReadSensordata+readSensordataIntervall)< actualTime:
          lastReadSensordata = actualTime
          data = readSensordata()
          
      htmlTable = createHtmlTable(data)
      website = getWebsite(htmlTable)
      deliverWebsite(website)
      
setup()
main()
